#include <SPI.h>
#include <MFRC522.h>
#include <LiquidCrystal_I2C.h>
#include <SD.h>
#include <Bounce2.h>

#define SS_PIN 10
#define RST_PIN 9
#define TASTE_PIN 2

//Sonderzeichen für LCD-Display
byte Glas[8] = {
  B11111,
  B10001,
  B10001,
  B10001,
  B01110,
  B00100,
  B00100,
  B11111
};

byte Gabel[8] = {
  B10101,
  B10101,
  B11111,
  B01110,
  B00100,
  B00100,
  B00100,
  B00100
};

byte Messer[8] = {
  B00110,
  B01110,
  B01110,
  B01110,
  B01110,
  B00010,
  B00010,
  B00010
};

MFRC522 mfrc522(SS_PIN, RST_PIN); // RFID-Empfänger

LiquidCrystal_I2C lcd(0x27, 20, 4);

Bounce2::Button Taste = Bounce2::Button();

String code;
unsigned long LesePause;
unsigned long AnzeigeDauer;
bool AnzeigeAktiv = 0;
bool ButtonPress = 0;

void setup() // Beginn des Setups:
{

  Serial.begin(9600); // Serieller Monitor zur Testanzeige
  SPI.begin(); // SPI-Verbindung aufbauen

  mfrc522.PCD_Init(); // Initialisierung des RFID-Empfängers

  Taste.attach( TASTE_PIN, INPUT_PULLUP );

  Serial.println("Start");
  Serial.println("Initialisiere SD-Karte");

  while (!SD.begin(4)) {
    Serial.println("Initialisierung fehlgeschlagen!");
    lcd.print("5252");
    while (1) {}
  }

  Serial.println("Initialisierung abgeschlossen");

  lcd.begin(); //Im Setup wird der LCD gestartet
  lcd.backlight(); //Hintergrundbeleuchtung einschalten (0 schaltet die Beleuchtung aus).

  lcd.createChar(0, Glas);
  lcd.createChar(1, Gabel);
  lcd.createChar(2, Messer);

  // Standardanzeige
  lcd.setCursor(0, 1);
  lcd.print("Anzeige");
  lcd.setCursor(0, 2);
  lcd.print("kurz vor Einfahrt");
}

//Function: Begriffe aus Textdatei lesen
String HoleTextfeld(String data, char separator, int index)
{
  int found = 0;
  int strIndex[] = {0, -1};
  int maxIndex = data.length() - 1;

  for (int i = 0; i <= maxIndex && found <= index; i++) {
    if (data.charAt(i) == separator || i == maxIndex) {
      found++;
      strIndex[0] = strIndex[1] + 1;
      strIndex[1] = (i == maxIndex) ? i + 1 : i;
    }
  }
  return found > index ? data.substring(strIndex[0], strIndex[1]) : "";
}

void loop()
{
  Taste.update();

  if ( Taste.pressed() ) {
    ButtonPress = 1;

    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Lesemodus");
  }

  if ( millis() > LesePause + 1000)
  {
    if ( mfrc522.PICC_IsNewCardPresent()) // Wenn Karte in Reichweite ist...
    {
      if ( mfrc522.PICC_ReadCardSerial())
      {
        String code;

        for (byte i = 0; i < mfrc522.uid.size; i++)
        {
          //Serial.print(mfrc522.uid.uidByte[i], HEX);

          if (String(mfrc522.uid.uidByte[i], HEX).length() == 1) {
            code = (code + "0");
          }

          code = (code + String(mfrc522.uid.uidByte[i], HEX));
        }

        Serial.print("UID: ");
        Serial.println(code);

        if ( ButtonPress == 1 ) {
          lcd.setCursor(0, 1);
          lcd.print(code);

          AnzeigeDauer = millis() + 3000;
          AnzeigeAktiv = 1;

          ButtonPress = 0;

        }
        else
        {

          String Datainame = String(code) + ".txt";

          File Textdatei;
          Textdatei = SD.open(Datainame, FILE_READ);

          if (Textdatei) {
            String Textinhalt;

            while (Textdatei.available()) {

              Textinhalt += char(Textdatei.read());
            }

            Textinhalt = sonderzeichen(Textinhalt);

            Serial.print("Textdateiinhalt: ");
            Serial.println(Textinhalt);

            lcd.clear();
            //Zeile 1
            lcd.setCursor(0, 0);
            lcd.print(HoleTextfeld(Textinhalt, ';', 0));
            lcd.print(" ");
            lcd.print(HoleTextfeld(Textinhalt, ';', 1));
            lcd.print(" ");
            lcd.print(HoleTextfeld(Textinhalt, ';', 2));
            
            //Zeile 2 - eventuellen überlauf von Zeile 1 mit Leerzeichen überschreiben
            lcd.setCursor(0, 1);
            lcd.print("                    ");
            
            //Zeile 3
            lcd.setCursor(0, 2);
            lcd.print(HoleTextfeld(Textinhalt, ';', 3));
            lcd.print(" ");
            lcd.print(HoleTextfeld(Textinhalt, ';', 4));
            lcd.print(" ");

            //Zeile 4 - Sondermeldung wenn nicht "0"
            String Meldung = HoleTextfeld(Textinhalt, ';', 5);
            Meldung.substring(0, 14);
            if (Meldung != "0") {
              lcd.setCursor(0, 3);
              lcd.print(Meldung);
            }
            
            //Zeile 4 - Catering-Anzeige - Positionierung nach Menge der Zeichen
            String Catering = HoleTextfeld(Textinhalt, ';', 6);
            if (Catering == "Glas") {
              lcd.setCursor(19, 3);
              lcd.write(byte(0));
            }
            else if (Catering == "Essen") {
              lcd.setCursor(18, 3);
              lcd.write(byte(1));
              lcd.write(byte(2));
            }

            //AnzeigeDauer in Sekunden
            AnzeigeDauer = millis() + HoleTextfeld(Textinhalt, ';', 7).toInt() * 1000;
            AnzeigeAktiv = 1;

            Serial.print("AnzeigeDauer in Millis: ");
            Serial.println(HoleTextfeld(Textinhalt, ';', 7).toInt() * 1000);

            Textinhalt = "";
          }
          else {
            Serial.println("Textdatei konnte nicht geoeffnet werden");
          }
          Textdatei.close();
        }
        LesePause = millis();
      }
    }
  }

  if ( (millis() >= AnzeigeDauer) && (AnzeigeAktiv == 1) ) {
    lcd.clear();
    lcd.setCursor(0, 1);
    lcd.print("Anzeige");
    lcd.setCursor(0, 2);
    lcd.print("kurz vor Einfahrt");
    AnzeigeAktiv = 0;
  }
}

//Function: Sonderzeichen ersetzten
String sonderzeichen( String input)
{
  input.replace("ä", "\341");
  input.replace("ß", "\342");
  input.replace("ö", "\357");
  input.replace("ü", "\365");
  return input;
}
